/*  libbadpenguin -- generic library for Bad Penguin development
 *
 *  Copyright (C) 1996 Antonio Gallo
 *  agx@linux.it
 *  Modified by Loren Lang, 9-26-01
 *  lorenl@tallye.com
 *  
 *  Newer versions should be on www.badpenguin.org
 *
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software Foundation,
 * Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.  
 */

#include <config.h>

#include <assert.h>
#include <dirent.h>
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <sys/stat.h>
#include <string.h>
#include <fcntl.h>

#include <libintl.h>
#define _(a)	dgettext(PACKAGE, a)

#include "badpenguin-ansi.h"
#include "badpenguin-errors.h"
#include "badpenguin-fs.h"


// Try to move a directory, first using the rename syscall
// if this file try to move it recursively
int move_dir( char* source, char* target) {
  struct dirent *lpstdirent;
  DIR *lpdir;
  struct stat lpstat;
  char sourcefile[256], targetfile[256];
    
  // Try to rename
  if ( 0 == rename(source,target) ) return 0;

  // Make target dir
  makedir(target, 0775 );
  // Scan target dir content
  lpdir = opendir(source);
  if (lpdir==NULL) {
    printERROR(_("move-dir: unable to open directory %s\n"), source );
    normal_color();
    return ERROR_IO;
  }
  lpstdirent = readdir(lpdir);
  while (lpstdirent!=NULL) {    
    if ((lpstdirent->d_name[0] == '.') && (lpstdirent->d_name[1] == 0) ) {
      lpstdirent = readdir(lpdir);
      continue;
    }
    if ((lpstdirent->d_name[0] == '.') && (lpstdirent->d_name[1] == '.') && (lpstdirent->d_name[2] == 0)) {
      lpstdirent = readdir(lpdir);
      continue;
    }
    // process item
    snprintf(sourcefile, 256, "%s/%s", source, lpstdirent->d_name);
    snprintf(targetfile, 256, "%s/%s", target, lpstdirent->d_name);    
    lstat(sourcefile, &lpstat);
    // Directory
    if (1 == S_ISDIR(lpstat.st_mode)) {
      move_dir(sourcefile,targetfile);
    } else
    // regular file
    if (1 == S_ISREG(lpstat.st_mode)) {
      move_file(sourcefile,targetfile);
    } else
    // symbolic-link
    if (1 == S_ISLNK(lpstat.st_mode)) {
      move_link(sourcefile,targetfile);
    } else
    // character or block device
    if (1 == S_ISBLK(lpstat.st_mode) || S_ISCHR(lpstat.st_mode)) {
      move_dev(sourcefile,targetfile);
    } else {
      printWARNING(_("move-dir: %s UNKNOWN! \n"), sourcefile);    
      normal_color();
    }
    // next file
    lpstdirent = readdir(lpdir);
  }
  closedir(lpdir);
  // apply permissions to target
  lstat( source, &lpstat );
  if ( 0 != file_time( target, lpstat.st_mtime, lpstat.st_atime ) ) {
    printERROR(_("move_dir: unable to change access and/or modification times for %s\n"), target);
    normal_color();      
  }  
  if ( 0 != chmod( target, lpstat.st_mode & 0xFFF ) ) {
    printERROR(_("move_dir: unable to change permissions for %s\n"), target);
    normal_color();
  }
  if ( 0 != chown( target, lpstat.st_uid, lpstat.st_gid )) {
    printERROR(_("move_dir: unable to set UID and/or GID for %s\n"), target);
    normal_color();        
  }
  // remove source    
  if ( 0 != rmdir(source) ) {
    printERROR(_("move_dir: unable to delete %s\n"), source);
    normal_color();        
  }
  return 0;
}


